# what file extensions should we monitor
source_ext = 'xjs'
exts = ['js', source_ext]
# TODO: support ignoring pre-minified and output files
sources = {}
source_dir = None
dest_dir = None
import_regex = '(//(\s*)(?P<command>import|require)) (\"|\')?(?P<script>\S+(.\S+)(.xjs|.js))(\"|\')?'
file_regex = '([a-zA-Z/]*.(?!min)(xjs|js))'
stack = []



import os
import config
import re

class Script:
    def __init__(self,filepath):
        self.parents = []
        self.path = filepath
        self.filename = os.path.basename(filepath)
        self.dir = os.path.dirname(filepath)
        self.extension = os.path.splitext(filepath)[1][1:]
        self.basename = os.path.splitext(os.path.split(filepath)[1])[0]
        self.parents = []



import re
import sys
import os
import time
import argparse
import parse
import config
from script import Script

try:
    from fsevents import Observer, Stream
except ImportError:
    from pyinotify import *


def main():
    cmd = sys.argv
    cmd.pop(0)
    """
    parse arguments and make go
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-s',
        '--src',
        help='source folder to watch',
        default='.',
        dest='src',
        metavar='folder'
    )
    parser.add_argument(
        '-d',
        '--dest',
        help='source folder to watch',
        default=None,
        dest='dest',
        metavar='folder'
    )
    args = parser.parse_args()
    print 'Initializing...'
    config.source_dir = os.path.abspath(args.src)
    if args.dest != None:
        config.dest_dir = os.path.abspath(args.dest)
    init_sources(config.source_dir)
    if cmd:
        c = cmd[0]
        commands = globals()
        if c in commands:
            commands[c]() 

def compile(): 
    for key,source in config.sources.iteritems():
        if source.extension == config.source_ext:
            parse.parse_file(source)

def watch():
    """
    initialize parameters and start the scanner
    """
    start_scanner(config.source_dir)

def dir_list(dir_name):
    outputList = []
    for root, dirs, files in os.walk(dir_name):
        outputList.append(root)
        for d in dirs:
            outputList.append(''.join([root, d]))
        for f1 in files:
            outputList.append('/'.join([root, f1]))
    return outputList

def init_sources(path):
    """
    initializes array of groups and their associated js files
    """
    for f in dir_list(path):
        if(os.path.splitext(f)[1][1:] == config.source_ext):
            print "Source file discovered: %s" % (f)
            script = Script(f)
            if (script.filename not in config.sources.keys()):
                config.sources[script.path] = script
                parse.parse_dependencies(script,script)

def start_scanner(path):
    """
    watch for file events in the supplied path
    """
    try:
        observer = Observer()
        observer.start()
        stream = Stream(file_modified, path, file_events=True)
        observer.schedule(stream)
        print "Watching for changes. Press Ctrl-C to stop."
        while 1:
          pass
    except (KeyboardInterrupt, OSError, IOError):
        observer.unschedule(stream)
        observer.stop()

def file_modified(event):
    """
    react to file events
    """
    if re.match(config.file_regex,event.name) or (event.name in config.sources.keys()):
        print "Change detected to: %s" % (event.name)
        config.stack = []
        script = config.sources[event.name]
        if script.extension == config.source_ext:
            parse.parse_file(script)
        else:
            parse.parse_parents(script)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

#pandoc -t rst -f markdown README.mkd -o README

import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='mass',
    version='0.1.4',
    description='Merge and Simplify Scripts: an automated tool for managing, combining and minifying javascript assets for web projects.',
    long_description=read('README'),
    author='jack boberg alex padgett',
    author_email='info@codedbyhand.com',
    url='https://github.com/coded-by-hand/mass',
    license='BSD License',
    platforms=['Mac OSX'],
    packages=['mass'],
    install_requires=['distribute','jsmin','macfsevents'],
    zip_safe = False,
    entry_points = {
        'console_scripts': [
            "mass = mass.monitor:main"
        ],
    }
)



import sys
from script import Script
import re
import jsmin
import os
import config

def parse_parents(src):
    for parent in src.parents:
        parse_file(parent)

def parse_file(src):
    """
    find file in config and output to dest dir
    """
    #clear the stack between parses
    if config.dest_dir == None:
        dest = src.dir
    else:
        dest = config.dest_dir
    output = get_output(src)
    output_file = dest + '/' + src.basename + '.min.js'
    f = open(output_file,'w')
    f.write(jsmin.jsmin(output))
    f.close()
    print "Wrote combined and minified file to: %s" % (output_file)

def get_output(src):
    """
    parse lines looking for commands
    """
    output = ''
    lines = open(src.path, 'rU').readlines()
    for line in lines:
        m = re.match(config.import_regex,line)
        if m:
            include_path = os.path.abspath(src.dir + '/' + m.group('script'));
            if include_path not in config.sources:
                script = Script(include_path)
                script.parents.append(src)
                config.sources[script.path] = script
            include_file = config.sources[include_path]
            #require statements dont include if the file has already been included
            if include_file not in config.stack or m.group('command') == 'import':
                config.stack.append(include_file)
                output += get_output(include_file)
        else:
            output += line
    return output

def execute(cmd, line):
    """
    get output from command
    """
    return

def parse_dependencies(script,context):
    lines = open(script.path).read()
    matches = re.finditer(config.import_regex,lines)
    for match in matches:
        # determine file path, recurse
        src = Script(os.path.abspath(script.dir + '/' + match.group('script')))
        if (src.path not in config.sources.keys()):
            src.parents.append(context)
            config.sources[src.path] = src
            parse_dependencies(src,context)
        elif (context not in config.sources[src.path].parents):
            config.sources[src.path].parents.append(context)
